/*
  RC5_SEND v1.0 1. Apr 2005 wb

  For a "F_CPU" AVR MCU
  
  Based on    http://www.mikrocontroller.net/forum/read-2-167641.html#new  by pebisoft
  
  IR led is on BP0 via a 100Ohm resistor to +5V VCC
  So LED is on if PB0 is set to 0, and LED is off if PB0 is set to 1.
  This is defined wrong in the original code.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <string.h>
#include <stdint.h>

// Timing Werte aus ASM-Listing ermittelt. Referenzfrequenz ist 8 MHz (8.0e6 Hz)
// F_CPU im Makefile definiert. 
#define DELAY_89ms      (uint32_t)(88997.875*(F_CPU/8.0e6)+0.499)
#define DELAY_HALFBIT   (uint16_t)(1184.019*(F_CPU/8.0e6)+0.499)
#define DELAY_36kHz     (uint8_t)(21.4*(F_CPU/8.0e6)+0.499)

#define IR_LED_DDR     DDRB
#define IR_LED_PORT    PORTB
#define IR_LED_BITNO   PB0
#define IR_LED_BIT     (1<<IR_LED_BITNO)

//#define ADVENTURE   /* Siehe Kommentar in send_bit */
#if defined(ADVENTURE)
  #define IR_LED_TOGGLE  (IR_LED_PORT ^= IR_LED_BIT)
#else
  #define IR_LED_ON      (IR_LED_PORT &= ~IR_LED_BIT)
  #define IR_LED_OFF     (IR_LED_PORT |= IR_LED_BIT)
  #define IR_LED_IS_OFF  ((IR_LED_PORT & IR_LED_BIT) != 0)
#endif

static void send_bit(uint8_t bit)
{
    uint16_t j;
    uint8_t i, counter;
    
    if(bit)
        for(j = 0; j < DELAY_HALFBIT; j++)
            asm volatile("nop" ::);
    for(i = 0; i < 64; i++)
    {
    #if defined (ADVENTURE) 
        IR_LED_TOGGLE;   // erzeugter code ist aber leider nicht atomar
                         // falls ein INT in der zwischenzeit eines der port-bits
                         // ndert kann es frchterlich knallen.
                         // Aber dann kann das sowiso nicht funktionieren
                         // weil ein interrupt das gesamte timing durcheinanderbringt.
                         // Also mssen eigentlich die Interrupts global disabled sein.
    #else                         
        // wird in bit-set/clear compiliert falls IO-register im unteren bereich                         
        (IR_LED_IS_OFF) ? (IR_LED_ON) : (IR_LED_OFF);
    #endif
        for(counter = 0; counter < DELAY_36kHz; counter++)
            asm volatile("nop" ::);
    }
    if(!bit)      
        for(j = 0; j < DELAY_HALFBIT; j++)
            asm volatile("nop" ::);
}

void rc5_send_code(uint8_t togglebit, uint8_t deviceaddress, uint8_t command)
{
    uint8_t mask = 0x10;
    uint32_t lcounter;
//    uint8_t sreg = SREG;
//    cli();
     
    // Sende 14 bit, dauert ca. 25 ms + 89ms Pause am ende
    send_bit(1);
    send_bit(1);
    send_bit(togglebit);
    while(mask)
    {
        send_bit(deviceaddress & mask);
        mask >>= 1;
    }
    mask = 0x20;
    while(mask)
    {
        send_bit(command & mask);
        mask >>= 1;
    }
//    SREG = sreg;
    // IR_LED_OFF; ist unnoetig da send_bit() am ende immer ausschaltet.
    // delay at least 89ms between codes
    for(lcounter = 0; lcounter < DELAY_89ms; lcounter++)
        asm volatile("nop" ::);
}


void rc5_send_init(void)
{
    IR_LED_DDR |= IR_LED_BIT;   // OUTPUT
    IR_LED_OFF;                 // PREPARED WITH LED OFF
}    
    

#if 0
int main(void)
{
    uint8_t k, i, cmd;

    rc5_send_init();
    
    cmd = 32;
    
    for(i = 14; i < cmd; i++)
    {
        for(k = 0; k < 32; k++)         // falls einer verlorengeht oder gestrt ist
            rc5_send_code(i & 1, 0x1F, i);   // toggles changes between every code 
    }
    
    while(1)
        ;
    return 0;
}
#endif